package edu.nyu.scps.weathergrid;

import android.os.AsyncTask;
import android.os.Bundle;
import android.support.v7.app.AppCompatActivity;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.widget.GridView;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;
import java.text.SimpleDateFormat;


public class MainActivity extends AppCompatActivity {

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        //Documentation about parameters such as q= and units= is at
        //http://openweathermap.org/current

        String urlString = "http://api.openweathermap.org/data/2.5/forecast/daily"
                + "?q=10279,US"     //The Woolworth Building has its own zip code.
                + "&cnt=7"          //number of days
                + "&units=imperial" //fahrenheit, not celsius
                + "&mode=json";     //vs. xml or html

        DownloadTask downloadTask = new DownloadTask();
        downloadTask.execute(urlString);
    }

    private class DownloadTask extends AsyncTask<String, Void, Void> {
        String[] date = new String[7]; //length of array must agree with cnt un URL
        int[] temperature = new int[7];
        int[] humidity = new int[7];

        //This method is executed by the second thread.

        @Override
        protected Void doInBackground(String... urlString) {
            String json;
            //Must be declared outside of try block,
            //so we can mention them in finally block.
            HttpURLConnection httpURLConnection = null;
            BufferedReader bufferedReader = null;

            try {
                URL url = new URL(urlString[0]);
                httpURLConnection = (HttpURLConnection) url.openConnection();
                httpURLConnection.setRequestMethod("GET");
                httpURLConnection.connect();

                InputStream inputStream = httpURLConnection.getInputStream();
                if (inputStream == null) {
                    return null;
                }

                StringBuffer buffer = new StringBuffer();
                InputStreamReader inputStreamReader = new InputStreamReader(inputStream);
                bufferedReader = new BufferedReader(inputStreamReader);

                String line;
                while ((line = bufferedReader.readLine()) != null) {
                    buffer.append(line);
                }

                if (buffer.length() == 0) {
                    return null;
                }
                json = buffer.toString();
            } catch (Exception exception) {
                Log.e("myTag", "doInBackground", exception);
                return null;
            } finally {
                if (httpURLConnection != null) {
                    httpURLConnection.disconnect();
                }
                if (bufferedReader != null) {
                    try {
                        bufferedReader.close();
                    } catch (final IOException exception) {
                        Log.e("myTag", "doInBackground", exception);
                    }
                }
            }

            //Parse the JSON string.
            try {
                JSONObject jSONObject = new JSONObject(json);
                JSONArray list = jSONObject.getJSONArray("list");
                SimpleDateFormat dateFormat = new SimpleDateFormat("E MMM d");

                for (int i = 0; i < list.length(); ++i) {
                    JSONObject day = (JSONObject)list.get(i);
                    long seconds = day.getLong("dt");
                    JSONObject temp = (JSONObject) day.getJSONObject("temp");
                    double max = temp.getDouble("max");
                    date[i] = dateFormat.format(1000L * seconds);
                    temperature[i] = (int)Math.round(max);
                    humidity[i] = day.getInt("humidity");
                }
            } catch (JSONException exception) {
                Log.e("myTag", "doInBackground", exception);
            }
            return null;
        }

        //This method is executed by the UI thread.

        @Override
        protected void onPostExecute(Void v) {
            ReportAdapter reportAdapter = new ReportAdapter(MainActivity.this, date, humidity, temperature);
            GridView gridView = (GridView)findViewById(R.id.gridView);
            gridView.setAdapter(reportAdapter);
        }
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_main, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();

        //noinspection SimplifiableIfStatement
        if (id == R.id.action_settings) {
            return true;
        }

        return super.onOptionsItemSelected(item);
    }
}
